@@ -45,6 +45,10 @@
 #endif
 #define SOEXT_LEN (strlen (SOEXT))
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
 static void
 print_escaped (const char *str)
 {
@@ -162,9 +166,24 @@ query_module (const char *dir, const char *file)
 		GdkPixbufFormat *info;
 		GdkPixbufModule *vtable;
 		
+#ifdef G_OS_WIN32
+		/* Replace backslashes in path with forward slashes, so that
+		 * it reads in without problems.
+		 */
+		{
+			char *p = path;
+			while (*p) {
+				if (*p == '\\')
+					*p = '/';
+				p++;
+			}
+		}
+#endif	
 		info = g_new0 (GdkPixbufFormat, 1);
 		vtable = g_new0 (GdkPixbufModule, 1);
-		
+
+		vtable->module = module;
+
 		(*fill_info) (info);
 		(*fill_vtable) (vtable);
 		
@@ -189,10 +208,60 @@ query_module (const char *dir, const char *file)
 int main (int argc, char **argv)
 {
 	gint i;
+	gchar *prgname;
 
+#ifdef G_OS_WIN32
+	gchar *libdir;
+	gchar *runtime_prefix;
+	gchar *slash;
+
+	if (g_ascii_strncasecmp (PIXBUF_LIBDIR, GTK_PREFIX, strlen (GTK_PREFIX)) == 0 &&
+	    G_IS_DIR_SEPARATOR (PIXBUF_LIBDIR[strlen (GTK_PREFIX)])) {
+		/* GTK_PREFIX is a prefix of PIXBUF_LIBDIR, as it
+		 * normally is. Replace that prefix in PIXBUF_LIBDIR
+		 * with the installation directory on this machine.
+		 * We assume this invokation of
+		 * gdk-pixbuf-query-loaders is run from either a "bin"
+		 * subdirectory of the installation directory, or in
+		 * the installation directory itself.
+		 */
+		if (G_WIN32_HAVE_WIDECHAR_API ()) {
+			wchar_t fn[1000];
+			GetModuleFileNameW (NULL, fn, G_N_ELEMENTS (fn));
+			runtime_prefix = g_utf16_to_utf8 (fn, -1, NULL, NULL, NULL);
+		}
+		else {
+			char fn[1000];
+			GetModuleFileNameA (NULL, fn, G_N_ELEMENTS (fn));
+			runtime_prefix = g_locale_to_utf8 (fn, -1, NULL, NULL, NULL);
+		}
+		slash = strrchr (runtime_prefix, '\\');
+		*slash = '\0';
+		slash = strrchr (runtime_prefix, '\\');
+		if (slash != NULL && g_ascii_strcasecmp (slash + 1, "bin") == 0) {
+			*slash = '\0';
+		}
+		
+		libdir = g_strconcat (runtime_prefix,
+				      "/",
+				      PIXBUF_LIBDIR + strlen (GTK_PREFIX) + 1,
+				      NULL);
+	}
+	else {
+		libdir = PIXBUF_LIBDIR;
+	}
+
+#undef PIXBUF_LIBDIR
+#define PIXBUF_LIBDIR libdir
+
+#endif
+	prgname = g_get_prgname ();
 	g_printf ("# GdkPixbuf Image Loader Modules file\n"
-		"# Automatically generated file, do not edit\n"
-		"#\n");
+		  "# Automatically generated file, do not edit\n"
+		  "# Created by %s from gtk+-%s\n"
+		  "#\n",
+		  (prgname ? prgname : "gdk-pixbuf-query-loaders"),
+		  GDK_PIXBUF_VERSION);
   
 	if (argc == 1) {
 #ifdef USE_GMODULE
@@ -200,6 +269,10 @@ int main (int argc, char **argv)
 		GDir *dir;
     
 		path = g_getenv ("GDK_PIXBUF_MODULEDIR");
+#ifdef G_OS_WIN32
+		if (path != NULL && *path != '\0')
+			path = g_locale_to_utf8 (path, -1, NULL, NULL, NULL);
+#endif
 		if (path == NULL || *path == '\0')
 			path = PIXBUF_LIBDIR;
 
@@ -225,9 +298,14 @@ int main (int argc, char **argv)
 	else {
 		char *cwd = g_get_current_dir ();
 
-		for (i = 1; i < argc; i++)
-			query_module (cwd, argv[i]);
-
+		for (i = 1; i < argc; i++) {
+			char *infilename = argv[i];
+#ifdef G_OS_WIN32
+			infilename = g_locale_to_utf8 (infilename,
+						       -1, NULL, NULL, NULL);
+#endif
+			query_module (cwd, infilename);
+		}
 		g_free (cwd);
 	}
 
