@@ -26,6 +26,7 @@
 
 #include <config.h>
 
+#include "gdkscreen.h"
 #include "gdkdisplay.h"
 #include "gdkdisplaymanager.h"
 
@@ -34,6 +35,8 @@
 
 #include "gdkintl.h"
 
+#include "gdkalias.h"
+
 struct _GdkDisplayManager
 {
   GObject parent_instance;
@@ -62,7 +65,7 @@ static void gdk_display_manager_get_property (GObject                *object,
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GdkDisplay *default_display = NULL;
+static GdkDisplay *default_display = NULL;
 
 GType
 gdk_display_manager_get_type (void)
@@ -100,6 +103,15 @@ gdk_display_manager_class_init (GdkDisplayManagerClass *klass)
   object_class->set_property = gdk_display_manager_set_property;
   object_class->get_property = gdk_display_manager_get_property;
 
+  /**
+   * GdkDisplayManager::display-opened:
+   * @display_manager: the object on which the signal is emitted
+   * @display: the opened display
+   *
+   * The ::display_opened signal is emitted when a display is opened.
+   *
+   * Since: 2.2
+   */
   signals[DISPLAY_OPENED] =
     g_signal_new ("display_opened",
 		  G_OBJECT_CLASS_TYPE (object_class),
@@ -114,8 +126,8 @@ gdk_display_manager_class_init (GdkDisplayManagerClass *klass)
   g_object_class_install_property (object_class,
 				   PROP_DEFAULT_DISPLAY,
 				   g_param_spec_object ("default-display",
- 							_("Default Display"),
- 							_("The default display for GDK"),
+ 							P_("Default Display"),
+ 							P_("The default display for GDK"),
 							GDK_TYPE_DISPLAY,
  							G_PARAM_READWRITE));
 }
@@ -247,7 +259,7 @@ gdk_display_manager_set_default_display (GdkDisplayManager *display_manager,
 
   _gdk_windowing_set_default_display (display);
 
-  g_object_notify (G_OBJECT (display_manager), "default_display");
+  g_object_notify (G_OBJECT (display_manager), "default-display");
 }
 
 /**
@@ -267,3 +279,5 @@ gdk_display_manager_list_displays (GdkDisplayManager *display_manager)
   return g_slist_copy (_gdk_displays);
 }
 
+#define __GDK_DISPLAY_MANAGER_C__
+#include "gdkaliasdef.c"
