@@ -25,7 +25,6 @@
  */
 
 #include <config.h>
-#include "gdkalias.h"
 #include "gdkwindow.h"
 #include "gdkinternals.h"
 #include "gdk.h"		/* For gdk_rectangle_union() */
@@ -33,6 +32,7 @@
 #include "gdkdrawable.h"
 #include "gdkpixmap.h"
 #include "gdkscreen.h"
+#include "gdkalias.h"
 
 #define USE_BACKING_STORE	/* Appears to work on Win32, too, now. */
 
@@ -105,12 +105,20 @@ static void   gdk_window_draw_lines     (GdkDrawable     *drawable,
 					 GdkGC           *gc,
 					 GdkPoint        *points,
 					 gint             npoints);
-static void   gdk_window_draw_glyphs    (GdkDrawable      *drawable,
-					 GdkGC            *gc,
-					 PangoFont        *font,
-					 gint              x,
-					 gint              y,
-					 PangoGlyphString *glyphs);
+
+static void gdk_window_draw_glyphs             (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
+static void gdk_window_draw_glyphs_transformed (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoMatrix      *matrix,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
 
 static void   gdk_window_draw_image     (GdkDrawable     *drawable,
                                          GdkGC           *gc,
@@ -135,6 +143,11 @@ static void gdk_window_draw_pixbuf (GdkDrawable     *drawable,
 				    gint             x_dither,
 				    gint             y_dither);
 
+static void gdk_window_draw_trapezoids (GdkDrawable   *drawable,
+					GdkGC	      *gc,
+					GdkTrapezoid  *trapezoids,
+					gint           n_trapezoids);
+
 static GdkImage* gdk_window_copy_to_image (GdkDrawable *drawable,
 					   GdkImage    *image,
 					   gint         src_x,
@@ -239,8 +252,10 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->draw_segments = gdk_window_draw_segments;
   drawable_class->draw_lines = gdk_window_draw_lines;
   drawable_class->draw_glyphs = gdk_window_draw_glyphs;
+  drawable_class->draw_glyphs_transformed = gdk_window_draw_glyphs_transformed;
   drawable_class->draw_image = gdk_window_draw_image;
   drawable_class->draw_pixbuf = gdk_window_draw_pixbuf;
+  drawable_class->draw_trapezoids = gdk_window_draw_trapezoids;
   drawable_class->get_depth = gdk_window_real_get_depth;
   drawable_class->get_screen = gdk_window_real_get_screen;
   drawable_class->get_size = gdk_window_real_get_size;
@@ -827,7 +842,7 @@ gdk_window_is_viewable (GdkWindow *window)
 	 (private != (GdkWindowObject *)root_window) &&
 	 (GDK_WINDOW_TYPE (private) != GDK_WINDOW_FOREIGN))
     {
-      if (GDK_WINDOW_DESTROYED (window) || !GDK_WINDOW_IS_MAPPED (window))
+      if (GDK_WINDOW_DESTROYED (private) || !GDK_WINDOW_IS_MAPPED (private))
 	return FALSE;
       
       private = (GdkWindowObject *)private->parent;
@@ -1656,6 +1671,51 @@ gdk_window_draw_glyphs (GdkDrawable      *drawable,
   RESTORE_GC (gc);
 }
 
+static void
+gdk_window_draw_glyphs_transformed (GdkDrawable      *drawable,
+				    GdkGC            *gc,
+				    PangoMatrix      *matrix,
+				    PangoFont        *font,
+				    gint              x,
+				    gint              y,
+				    PangoGlyphString *glyphs)
+{
+  GdkWindowObject *private = (GdkWindowObject *)drawable;
+  PangoMatrix tmp_matrix;
+
+  OFFSET_GC (gc);
+
+  if (GDK_WINDOW_DESTROYED (drawable))
+    return;
+
+  if (x_offset != 0 || y_offset != 0)
+    {
+      if (matrix)
+	{
+	  tmp_matrix = *matrix;
+	  tmp_matrix.x0 -= x_offset;
+	  tmp_matrix.y0 -= y_offset;
+	  matrix = &tmp_matrix;
+	}
+      else
+	{
+	  x -= x_offset * PANGO_SCALE;
+	  y -= y_offset * PANGO_SCALE;
+	}
+    }
+  
+  if (private->paint_stack)
+    {
+      GdkWindowPaint *paint = private->paint_stack->data;
+
+      gdk_draw_glyphs_transformed (paint->pixmap, gc, matrix, font, x, y, glyphs);
+    }
+  else
+    gdk_draw_glyphs_transformed (private->impl, gc, matrix, font, x, y, glyphs);
+
+  RESTORE_GC (gc);
+}
+
 static GdkGC *
 gdk_window_get_bg_gc (GdkWindow      *window,
 		      GdkWindowPaint *paint)
@@ -1895,6 +1955,52 @@ gdk_window_draw_pixbuf (GdkDrawable     *drawable,
     }
 }
 
+static void
+gdk_window_draw_trapezoids (GdkDrawable   *drawable,
+			    GdkGC	  *gc,
+			    GdkTrapezoid  *trapezoids,
+			    gint           n_trapezoids)
+{
+  GdkWindowObject *private = (GdkWindowObject *)drawable;
+  GdkTrapezoid *new_trapezoids = NULL;
+
+  OFFSET_GC (gc);
+
+  if (GDK_WINDOW_DESTROYED (drawable))
+    return;
+  
+  if (x_offset != 0 || y_offset != 0)
+    {
+      gint i;
+
+      new_trapezoids = g_new (GdkTrapezoid, n_trapezoids);
+      for (i=0; i < n_trapezoids; i++)
+	{
+	  new_trapezoids[i].y1 = trapezoids[i].y1 - y_offset;
+	  new_trapezoids[i].x11 = trapezoids[i].x11 - x_offset;
+	  new_trapezoids[i].x21 = trapezoids[i].x21 - x_offset;
+	  new_trapezoids[i].y2 = trapezoids[i].y2 - y_offset;
+	  new_trapezoids[i].x12 = trapezoids[i].x12 - x_offset;
+	  new_trapezoids[i].x22 = trapezoids[i].x22 - x_offset;
+	}
+
+      trapezoids = new_trapezoids;
+    }
+
+  if (private->paint_stack)
+    {
+      GdkWindowPaint *paint = private->paint_stack->data;
+      gdk_draw_trapezoids (paint->pixmap, gc, trapezoids, n_trapezoids);
+    }
+  else
+    gdk_draw_trapezoids (private->impl, gc, trapezoids, n_trapezoids);
+  
+  if (new_trapezoids)
+    g_free (new_trapezoids);
+
+  RESTORE_GC (gc);
+}
+
 static void
 gdk_window_real_get_size (GdkDrawable *drawable,
                           gint *width,
@@ -2062,32 +2168,20 @@ gdk_window_process_updates_internal (GdkWindow *window)
 				window_region);
 	  gdk_region_destroy (window_region);
 	  
-	  if (!gdk_region_empty (expose_region))
+	  if (!gdk_region_empty (expose_region) &&
+	      (private->event_mask & GDK_EXPOSURE_MASK))
 	    {
-	      if (private->event_mask & GDK_EXPOSURE_MASK)
-		{
-		  GdkEvent event;
+	      GdkEvent event;
 	      
-		  event.expose.type = GDK_EXPOSE;
-		  event.expose.window = g_object_ref (window);
-		  event.expose.count = 0;
-		  event.expose.region = expose_region;
-		  gdk_region_get_clipbox (expose_region, &event.expose.area);
-		  
-		  (*_gdk_event_func) (&event, _gdk_event_data);
-
-		  g_object_unref (window);
-		}
-	      else
-		{
-		  /* Windows without GDK_EXPOSURE_MASK still need to
-		   * get their backgrounds painted, because it
-		   * may have been temporarily set to None when
-		   * the expose was generated
-		   */
-		  gdk_window_begin_paint_region (window, expose_region);
-		  gdk_window_end_paint (window);
-		}
+	      event.expose.type = GDK_EXPOSE;
+	      event.expose.window = g_object_ref (window);
+	      event.expose.count = 0;
+	      event.expose.region = expose_region;
+	      gdk_region_get_clipbox (expose_region, &event.expose.area);
+	      
+	      (*_gdk_event_func) (&event, _gdk_event_data);
+	      
+	      g_object_unref (window);
 	    }
 
 	  if (expose_region != update_area)
@@ -2765,3 +2859,5 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   return gdk_window_foreign_new_for_display (gdk_display_get_default (), anid);
 }
 
+#define __GDK_WINDOW_C__
+#include "gdkaliasdef.c"
