@@ -25,6 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -353,6 +354,7 @@ alloc_color (GdkColormap  *cmap,
       return TRUE;
 
     default:
+      g_error ("cmap->visual->type=%d", cmap->visual->type);
       g_assert_not_reached ();
       return FALSE;
     }
