@@ -25,6 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -1017,9 +1018,9 @@ gdk_font_load_internal (const gchar *font_name)
 	if (strcmp (point_size, "*") == 0)
 	  logfont.lfHeight = 0;
 	else
-	  logfont.lfHeight = (int) (((double) atoi (point_size))/720.*logpixelsy);
+	  logfont.lfHeight = -(int) (((double) atoi (point_size))/720.*logpixelsy);
       else
-	logfont.lfHeight = atoi (pixel_size);
+	logfont.lfHeight = -atoi (pixel_size);
 
       logfont.lfWidth = 0;
       logfont.lfEscapement = 0;
@@ -1591,8 +1592,8 @@ gdk_text_extents (GdkFont     *font,
 		  gint        *descent)
 {
   gdk_text_size_arg arg;
-  gint wlen;
-  wchar_t *wcstr;
+  glong wlen;
+  wchar_t *wcstr, wc;
 
   g_return_if_fail (font != NULL);
   g_return_if_fail (text != NULL);
@@ -1616,22 +1617,18 @@ gdk_text_extents (GdkFont     *font,
 
   arg.total.cx = arg.total.cy = 0;
 
-  wcstr = g_new (wchar_t, text_length);
   if (text_length == 1)
     {
-      wcstr[0] = (guchar) text[0];
-      _gdk_wchar_text_handle (font, wcstr, 1, gdk_text_size_handler, &arg);
+      wc = (guchar) text[0];
+      _gdk_wchar_text_handle (font, &wc, 1, gdk_text_size_handler, &arg);
     }
   else
     {
-      if ((wlen = _gdk_utf8_to_ucs2 (wcstr, text, text_length, text_length)) == -1)
-	g_warning ("gdk_text_extents: _gdk_utf8_to_ucs2 failed");
-      else
-	_gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, &arg);
+      wcstr = g_utf8_to_utf16 (text, text_length, NULL, &wlen, NULL);
+      _gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, &arg);
+      g_free (wcstr);
     }
 
-  g_free (wcstr);
-
   /* XXX This is quite bogus */
   if (lbearing)
     *lbearing = 0;
@@ -1709,3 +1706,9 @@ gdk_text_extents_wc (GdkFont        *font,
   if (descent)
     *descent = font->descent + 1;
 }
+
+GdkDisplay* 
+gdk_font_get_display (GdkFont* font)
+{
+  return _gdk_display;
+}
