@@ -119,6 +119,115 @@ gdk_gc_win32_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+fixup_pen (GdkGCWin32 *win32_gc)
+{
+  win32_gc->pen_style = 0;
+
+  /* First look at GDK width and end cap style, set GDI pen type and
+   * end cap.
+   */
+  if (win32_gc->pen_width == 0 &&
+      win32_gc->cap_style == GDK_CAP_NOT_LAST)
+    {
+      /* Use a cosmetic pen, always width 1 */
+      win32_gc->pen_style |= PS_COSMETIC;
+    }
+  else if (win32_gc->pen_width <= 1 &&
+	   win32_gc->cap_style == GDK_CAP_BUTT)
+    {
+      /* For 1 pixel wide lines PS_ENDCAP_ROUND means draw both ends,
+       * even for one pixel length lines. But if we are drawing dashed
+       * lines we can't use PS_ENDCAP_ROUND.
+       */
+      if (win32_gc->line_style == GDK_LINE_SOLID)
+	win32_gc->pen_style |= PS_GEOMETRIC | PS_ENDCAP_ROUND;
+      else
+	win32_gc->pen_style |= PS_GEOMETRIC | PS_ENDCAP_FLAT;
+    }
+  else
+    {
+      win32_gc->pen_style |= PS_GEOMETRIC;
+      switch (win32_gc->cap_style)
+	{
+	/* For non-zero-width lines X11's CapNotLast works like CapButt */
+	case GDK_CAP_NOT_LAST:
+	case GDK_CAP_BUTT:
+	  win32_gc->pen_style |= PS_ENDCAP_FLAT;
+	  break;
+	case GDK_CAP_ROUND:
+	  win32_gc->pen_style |= PS_ENDCAP_ROUND;
+	  break;
+	case GDK_CAP_PROJECTING:
+	  win32_gc->pen_style |= PS_ENDCAP_SQUARE;
+	  break;
+	}
+    }
+
+  /* Next look at GDK line style, set GDI pen style attribute */
+  switch (win32_gc->line_style)
+    {
+    case GDK_LINE_SOLID:
+      win32_gc->pen_style |= PS_SOLID;
+      break;
+    case GDK_LINE_ON_OFF_DASH:
+    case GDK_LINE_DOUBLE_DASH:
+      if (win32_gc->pen_dashes == NULL)
+	{
+	  win32_gc->pen_dashes = g_new (DWORD, 1);
+	  win32_gc->pen_dashes[0] = 4;
+	  win32_gc->pen_num_dashes = 1;
+	}
+      if (G_WIN32_IS_NT_BASED ())
+	{
+	  if (!(win32_gc->pen_style & PS_TYPE_MASK) == PS_GEOMETRIC &&
+	      win32_gc->pen_dashes[0] == 1 &&
+	      (win32_gc->pen_num_dashes == 1 ||
+	       (win32_gc->pen_num_dashes == 2 && win32_gc->pen_dashes[0] == 1)))
+	    win32_gc->pen_style |= PS_ALTERNATE;
+	  else
+	    win32_gc->pen_style |= PS_USERSTYLE;
+	}
+      else
+	{
+	  /* Render "short" on-off dashes drawn with R2_COPYPEN and a
+	   * cosmetic pen using PS_DOT
+	   */
+	  if (win32_gc->line_style == GDK_LINE_ON_OFF_DASH &&
+	      win32_gc->rop2 == R2_COPYPEN &&
+	      (win32_gc->pen_style & PS_TYPE_MASK) == PS_COSMETIC &&
+	      win32_gc->pen_dashes[0] <= 2 &&
+	      (win32_gc->pen_num_dashes == 1 ||
+	       (win32_gc->pen_num_dashes == 2 && win32_gc->pen_dashes[1] <= 2)))
+	    win32_gc->pen_style |= PS_DOT;
+	  else
+	    /* Otherwise render opaque lines solid, horizontal or
+	     * vertical ones will be dashed manually, see
+	     * gdkdrawable-win32.c.
+	     */
+	    win32_gc->pen_style |= PS_SOLID;
+	}
+     break;
+    }
+
+  /* Last, for if the GDI pen is geometric, set the join attribute */
+  if ((win32_gc->pen_style & PS_TYPE_MASK) == PS_GEOMETRIC)
+    {
+      switch (win32_gc->join_style)
+	{
+	case GDK_JOIN_MITER:
+	  win32_gc->pen_style |= PS_JOIN_MITER;
+	  break;
+	case GDK_JOIN_ROUND:
+	  win32_gc->pen_style |= PS_JOIN_ROUND;
+	  break;
+	case GDK_JOIN_BEVEL:
+	  win32_gc->pen_style |= PS_JOIN_BEVEL;
+	  break;
+	}
+    }
+}
+
 static void
 gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 				    GdkGCValuesMask mask,
@@ -355,76 +464,32 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 
   if (mask & GDK_GC_LINE_STYLE)
     {
-      switch (values->line_style)
-	{
-	case GDK_LINE_SOLID:
-	  if (win32_gc->pen_dashes)
-	    {
-	      g_free (win32_gc->pen_dashes);
-	      win32_gc->pen_dashes = NULL;
-	      win32_gc->pen_num_dashes = 0;
-	    }
-          win32_gc->pen_style &= ~(PS_STYLE_MASK);
-	  win32_gc->pen_style |= PS_SOLID;
-	  break;
-	case GDK_LINE_ON_OFF_DASH:
-	case GDK_LINE_DOUBLE_DASH: 
-	  if (!win32_gc->pen_dashes)
-	    {
-            /* setting to PS_DASH probably isn't correct. If I understand the
-             * xlib docs correctly it should influence the handling of
-             * line endings ? --hb
-             */
-            win32_gc->pen_style &= ~(PS_STYLE_MASK);
-	    win32_gc->pen_style |= PS_DASH;
-          }
-	  break;
-	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_STYLE_%s", s, _gdk_win32_psstyle_to_string (win32_gc->pen_style)),
-		     s = ","));
+      win32_gc->line_style = values->line_style;
       win32_gc->values_mask |= GDK_GC_LINE_STYLE;
     }
 
   if (mask & GDK_GC_CAP_STYLE)
     {
-      win32_gc->pen_style &= ~(PS_ENDCAP_MASK);
-      switch (values->cap_style)
-	{
-	case GDK_CAP_NOT_LAST:	/* ??? */
-	case GDK_CAP_BUTT:
-	  win32_gc->pen_style |= PS_ENDCAP_FLAT;
-	  break;
-	case GDK_CAP_ROUND:
-	  win32_gc->pen_style |= PS_ENDCAP_ROUND;
-	  break;
-	case GDK_CAP_PROJECTING:
-	  win32_gc->pen_style |= PS_ENDCAP_SQUARE;
-	  break;
-	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_ENDCAP_%s", s, _gdk_win32_psendcap_to_string (win32_gc->pen_style)),
-		     s = ","));
+      win32_gc->cap_style = values->cap_style;
       win32_gc->values_mask |= GDK_GC_CAP_STYLE;
     }
 
   if (mask & GDK_GC_JOIN_STYLE)
     {
-      win32_gc->pen_style &= ~(PS_JOIN_MASK);
-      switch (values->join_style)
-	{
-	case GDK_JOIN_MITER:
-	  win32_gc->pen_style |= PS_JOIN_MITER;
-	  break;
-	case GDK_JOIN_ROUND:
-	  win32_gc->pen_style |= PS_JOIN_ROUND;
-	  break;
-	case GDK_JOIN_BEVEL:
-	  win32_gc->pen_style |= PS_JOIN_BEVEL;
-	  break;
-	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_JOIN_%s", s, _gdk_win32_psjoin_to_string (win32_gc->pen_style)),
-		     s = ","));
+      win32_gc->join_style = values->join_style;
       win32_gc->values_mask |= GDK_GC_JOIN_STYLE;
     }
+
+  if (mask & (GDK_GC_LINE_WIDTH|GDK_GC_LINE_STYLE|GDK_GC_CAP_STYLE|GDK_GC_JOIN_STYLE))
+    {
+      fixup_pen (win32_gc);
+      GDK_NOTE (GC, (g_print ("%sps|=PS_STYLE_%s|PS_ENDCAP_%s|PS_JOIN_%s", s,
+			      _gdk_win32_psstyle_to_string (win32_gc->pen_style),
+			      _gdk_win32_psendcap_to_string (win32_gc->pen_style),
+			      _gdk_win32_psjoin_to_string (win32_gc->pen_style)),
+		     s = ","));
+    }
+
   GDK_NOTE (GC, g_print ("} mask=(%s)", _gdk_win32_gcvalues_mask_to_string (win32_gc->values_mask)));
 }
 
@@ -459,9 +524,18 @@ _gdk_win32_gc_new (GdkDrawable	  *drawable,
   win32_gc->subwindow_mode = GDK_CLIP_BY_CHILDREN;
   win32_gc->graphics_exposures = TRUE;
   win32_gc->pen_width = 0;
-  win32_gc->pen_style = PS_GEOMETRIC|PS_ENDCAP_FLAT|PS_JOIN_MITER;
+  /* Don't get confused by the PS_ENDCAP_ROUND. For narrow GDI pens
+   * (width == 1), PS_GEOMETRIC|PS_ENDCAP_ROUND works like X11's
+   * CapButt.
+   */
+  win32_gc->pen_style = PS_GEOMETRIC|PS_ENDCAP_ROUND|PS_JOIN_MITER;
+  win32_gc->line_style = GDK_LINE_SOLID;
+  win32_gc->cap_style = GDK_CAP_BUTT;
+  win32_gc->join_style = GDK_JOIN_MITER;
   win32_gc->pen_dashes = NULL;
   win32_gc->pen_num_dashes = 0;
+  win32_gc->pen_dash_offset = 0;
+  win32_gc->pen_hbrbg = NULL;
 
   win32_gc->values_mask = GDK_GC_FUNCTION | GDK_GC_FILL;
 
@@ -537,28 +611,9 @@ gdk_win32_gc_get_values (GdkGC       *gc,
   values->graphics_exposures = win32_gc->graphics_exposures;
   values->line_width = win32_gc->pen_width;
   
-  if (win32_gc->pen_style & PS_SOLID)
-    values->line_style = GDK_LINE_SOLID;
-  else if (win32_gc->pen_style & PS_DASH)
-    values->line_style = GDK_LINE_ON_OFF_DASH;
-  else
-    values->line_style = GDK_LINE_SOLID;
-
-  /* PS_ENDCAP_ROUND is zero */
-  if (win32_gc->pen_style & PS_ENDCAP_FLAT)
-    values->cap_style = GDK_CAP_BUTT;
-  else if (win32_gc->pen_style & PS_ENDCAP_SQUARE)
-    values->cap_style = GDK_CAP_PROJECTING;
-  else
-    values->cap_style = GDK_CAP_ROUND;
-    
-  /* PS_JOIN_ROUND is zero */
-  if (win32_gc->pen_style & PS_JOIN_MITER)
-    values->join_style = GDK_JOIN_MITER;
-  else if (win32_gc->pen_style & PS_JOIN_BEVEL)
-    values->join_style = GDK_JOIN_BEVEL;
-  else
-    values->join_style = GDK_JOIN_ROUND;
+  values->line_style = win32_gc->line_style;
+  values->cap_style = win32_gc->cap_style;
+  values->join_style = win32_gc->join_style;
 }
 
 static void
@@ -587,18 +642,13 @@ gdk_win32_gc_set_dashes (GdkGC *gc,
 
   win32_gc = GDK_GC_WIN32 (gc);
 
-  /* mark as set, see gdk_win32_gc_values_to_win32values () for the reason */
-  win32_gc->values_mask |= GDK_GC_LINE_STYLE;
-
-  win32_gc->pen_style &= ~(PS_STYLE_MASK);
-
-  win32_gc->pen_style |= (PS_GEOMETRIC | PS_USERSTYLE);
   win32_gc->pen_num_dashes = n;
-  if (win32_gc->pen_dashes != NULL)
-    g_free (win32_gc->pen_dashes);
+  g_free (win32_gc->pen_dashes);
   win32_gc->pen_dashes = g_new (DWORD, n);
   for (i = 0; i < n; i++)
     win32_gc->pen_dashes[i] = dash_list[i];
+  win32_gc->pen_dash_offset = dash_offset;
+  fixup_pen (win32_gc);
 }
 
 void
@@ -746,17 +796,23 @@ gdk_gc_copy (GdkGC *dst_gc,
   dst_win32_gc->graphics_exposures = src_win32_gc->graphics_exposures;
   dst_win32_gc->pen_width = src_win32_gc->pen_width;
   dst_win32_gc->pen_style = src_win32_gc->pen_style;
-  dst_win32_gc->pen_dashes = src_win32_gc->pen_dashes;
-  if (dst_win32_gc->pen_dashes)
+  dst_win32_gc->line_style = src_win32_gc->line_style;
+  dst_win32_gc->cap_style = src_win32_gc->cap_style;
+  dst_win32_gc->join_style = src_win32_gc->join_style;
+  if (src_win32_gc->pen_dashes)
     dst_win32_gc->pen_dashes = g_memdup (src_win32_gc->pen_dashes, 
                                          sizeof (DWORD) * src_win32_gc->pen_num_dashes);
+  else
+    dst_win32_gc->pen_dashes = NULL;
   dst_win32_gc->pen_num_dashes = src_win32_gc->pen_num_dashes;
+  dst_win32_gc->pen_dash_offset = src_win32_gc->pen_dash_offset;
 
 
   dst_win32_gc->hdc = NULL;
   dst_win32_gc->saved_dc = FALSE;
   dst_win32_gc->hwnd = NULL;
   dst_win32_gc->holdpal = NULL;
+  dst_win32_gc->pen_hbrbg = NULL;
 }
 
 GdkScreen *  
@@ -807,15 +863,14 @@ _gdk_win32_colormap_color (GdkColormap *colormap,
     }
 }
 
-static COLORREF
-predraw_set_foreground (GdkGC       *gc,
-			GdkColormap *colormap,
-			gboolean    *ok)
+gboolean
+predraw (GdkGC       *gc,
+	 GdkColormap *colormap)
 {
-  COLORREF fg;
   GdkGCWin32 *win32_gc = (GdkGCWin32 *) gc;
   GdkColormapPrivateWin32 *colormap_private;
   gint k;
+  gboolean ok = TRUE;
 
   if (colormap &&
       (colormap->visual->type == GDK_VISUAL_PSEUDO_COLOR ||
@@ -826,18 +881,15 @@ predraw_set_foreground (GdkGC       *gc,
       g_assert (colormap_private != NULL);
 
       if (!(win32_gc->holdpal = SelectPalette (win32_gc->hdc, colormap_private->hpal, FALSE)))
-	WIN32_GDI_FAILED ("SelectPalette"), *ok = FALSE;
+	WIN32_GDI_FAILED ("SelectPalette"), ok = FALSE;
       else if ((k = RealizePalette (win32_gc->hdc)) == GDI_ERROR)
-	WIN32_GDI_FAILED ("RealizePalette"), *ok = FALSE;
+	WIN32_GDI_FAILED ("RealizePalette"), ok = FALSE;
       else if (k > 0)
-	GDK_NOTE (COLORMAP, g_print ("predraw_set_foreground: realized %p: %d colors\n",
+	GDK_NOTE (COLORMAP, g_print ("predraw: realized %p: %d colors\n",
 				     colormap_private->hpal, k));
     }
 
-  fg = _gdk_win32_colormap_color (colormap, win32_gc->foreground);
-
-  GDK_NOTE (GC, g_print ("predraw_set_foreground: fg=%06lx\n", fg));
-  return fg;
+  return ok;
 }
 
 /**
@@ -898,8 +950,7 @@ gdk_win32_hdc_get (GdkDrawable    *drawable,
   GdkGCWin32 *win32_gc = (GdkGCWin32 *) gc;
   GdkDrawableImplWin32 *impl = NULL;
   gboolean ok = TRUE;
-  COLORREF fg = RGB (0, 0, 0);
-  LOGBRUSH logbrush;
+  COLORREF fg = RGB (0, 0, 0), bg = RGB (255, 255, 255);
   HPEN hpen;
   HBRUSH hbr;
 
@@ -935,11 +986,14 @@ gdk_win32_hdc_get (GdkDrawable    *drawable,
       if (ok && (win32_gc->saved_dc = SaveDC (win32_gc->hdc)) == 0)
 	WIN32_GDI_FAILED ("SaveDC");
     }
-  
+
+  if (ok && (usage & (GDK_GC_FOREGROUND | GDK_GC_BACKGROUND)))
+      ok = predraw (gc, impl->colormap);
+
   if (ok && (usage & GDK_GC_FOREGROUND))
     {
-      fg = predraw_set_foreground (gc, impl->colormap, &ok);
-      if (ok && (hbr = CreateSolidBrush (fg)) == NULL)
+      fg = _gdk_win32_colormap_color (impl->colormap, win32_gc->foreground);
+      if ((hbr = CreateSolidBrush (fg)) == NULL)
 	WIN32_GDI_FAILED ("CreateSolidBrush"), ok = FALSE;
 
       if (ok && SelectObject (win32_gc->hdc, hbr) == NULL)
@@ -951,36 +1005,40 @@ gdk_win32_hdc_get (GdkDrawable    *drawable,
 
   if (ok && (usage & LINE_ATTRIBUTES))
     {
-      /* Create and select pen */
-      logbrush.lbStyle = BS_SOLID;
-      logbrush.lbColor = fg;
-      logbrush.lbHatch = 0;
-      
-      if (win32_gc->pen_num_dashes > 0 && !G_WIN32_IS_NT_BASED ())
-	{
-	  /* The Win9x GDI is rather limited so we either draw dashed
-	   * lines ourselves (only horizontal and vertical) or let them be
-	   * drawn solid to avoid implementing a whole line renderer.
-	   */
-	  if ((hpen = ExtCreatePen (
-				    (win32_gc->pen_style & ~(PS_STYLE_MASK)) | PS_SOLID,
-				    MAX (win32_gc->pen_width, 1),
-				    &logbrush, 
-				    0, NULL)) == NULL)
-	    WIN32_GDI_FAILED ("ExtCreatePen"), ok = FALSE;
-	}
-      else
-	{
+      /* For drawing GDK_LINE_DOUBLE_DASH */
+      if ((usage & GDK_GC_BACKGROUND) && win32_gc->line_style == GDK_LINE_DOUBLE_DASH)
+        {
+          bg = _gdk_win32_colormap_color (impl->colormap, win32_gc->background);
+          if ((win32_gc->pen_hbrbg = CreateSolidBrush (bg)) == NULL)
+	    WIN32_GDI_FAILED ("CreateSolidBrush"), ok = FALSE;
+        }
+
+      if (ok)
+        {
+	  LOGBRUSH logbrush;
+	  DWORD style_count = 0;
+	  const DWORD *style = NULL;
+
+	  /* Create and select pen */
+	  logbrush.lbStyle = BS_SOLID;
+	  logbrush.lbColor = fg;
+	  logbrush.lbHatch = 0;
+
+	  if ((win32_gc->pen_style & PS_STYLE_MASK) == PS_USERSTYLE)
+	    {
+	      style_count = win32_gc->pen_num_dashes;
+	      style = win32_gc->pen_dashes;
+	    }
+
 	  if ((hpen = ExtCreatePen (win32_gc->pen_style,
 				    MAX (win32_gc->pen_width, 1),
 				    &logbrush, 
-				    win32_gc->pen_num_dashes,
-				    win32_gc->pen_dashes)) == NULL)
+				    style_count, style)) == NULL)
 	    WIN32_GDI_FAILED ("ExtCreatePen"), ok = FALSE;
+	  
+	  if (ok && SelectObject (win32_gc->hdc, hpen) == NULL)
+	    WIN32_GDI_FAILED ("SelectObject"), ok = FALSE;
 	}
-      
-      if (ok && SelectObject (win32_gc->hdc, hpen) == NULL)
-	WIN32_GDI_FAILED ("SelectObject"), ok = FALSE;
     }
 
   if (ok && (usage & GDK_GC_FONT))
@@ -1085,6 +1143,9 @@ gdk_win32_hdc_release (GdkDrawable    *drawable,
   if (hbr != NULL)
     GDI_CALL (DeleteObject, (hbr));
 
+  if (win32_gc->pen_hbrbg != NULL)
+    GDI_CALL (DeleteObject, (win32_gc->pen_hbrbg));
+
   win32_gc->hdc = NULL;
 }
 
@@ -1229,20 +1290,16 @@ _gdk_win32_gdkregion_to_hrgn (GdkRegion *region,
   rgndata->rdh.iType = RDH_RECTANGLES;
   rgndata->rdh.nCount = rgndata->rdh.nRgnSize = 0;
   SetRect (&rgndata->rdh.rcBound,
-	   G_MAXSHORT, G_MAXSHORT, G_MINSHORT, G_MINSHORT);
+	   G_MAXLONG, G_MAXLONG, G_MINLONG, G_MINLONG);
 
   for (i = 0; i < region->numRects; i++)
     {
       rect = ((RECT *) rgndata->Buffer) + rgndata->rdh.nCount++;
 
-      rect->left = CLAMP (boxes[i].x1 + x_origin,
-			  G_MINSHORT, G_MAXSHORT);
-      rect->right = CLAMP (boxes[i].x2 + x_origin,
-			   G_MINSHORT, G_MAXSHORT);
-      rect->top = CLAMP (boxes[i].y1 + y_origin,
-			 G_MINSHORT, G_MAXSHORT);
-      rect->bottom = CLAMP (boxes[i].y2 + y_origin,
-			    G_MINSHORT, G_MAXSHORT);
+      rect->left = boxes[i].x1 + x_origin;
+      rect->right = boxes[i].x2 + x_origin;
+      rect->top = boxes[i].y1 + y_origin;
+      rect->bottom = boxes[i].y2 + y_origin;
 
       if (rect->left < rgndata->rdh.rcBound.left)
 	rgndata->rdh.rcBound.left = rect->left;
