@@ -30,7 +30,6 @@
 
 #include <config.h>
 
-#include "gtkalias.h"
 #include "gtkaction.h"
 #include "gtkactiongroup.h"
 #include "gtkaccellabel.h"
@@ -45,6 +44,7 @@
 #include "gtktearoffmenuitem.h"
 #include "gtktoolbutton.h"
 #include "gtktoolbar.h"
+#include "gtkalias.h"
 
 
 #define GTK_ACTION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_ACTION, GtkActionPrivate))
@@ -446,7 +446,7 @@ gtk_action_set_property (GObject         *object,
 	  tmp = action->private_data->short_label;
 	  action->private_data->short_label = g_strdup (action->private_data->label);
 	  g_free (tmp);
-	  g_object_notify (object, "short_label");
+	  g_object_notify (object, "short-label");
 	}
       break;
     case PROP_SHORT_LABEL:
@@ -486,7 +486,7 @@ gtk_action_set_property (GObject         *object,
 	  tmp = action->private_data->short_label;
 	  action->private_data->short_label = g_strdup (action->private_data->label);
 	  g_free (tmp);
-	  g_object_notify (object, "short_label");
+	  g_object_notify (object, "short-label");
 	}
       break;
     case PROP_VISIBLE_HORIZONTAL:
@@ -888,6 +888,8 @@ connect_proxy (GtkAction     *action,
     }
   else if (GTK_IS_TOOL_ITEM (proxy))
     {
+      GParamSpec *pspec;
+
       /* toolbar item specific synchronisers ... */
 
       g_object_set (proxy,
@@ -895,9 +897,11 @@ connect_proxy (GtkAction     *action,
 		    "visible_vertical",	  action->private_data->visible_vertical,
 		    "is_important", action->private_data->is_important,
 		    NULL);
-      /* FIXME: we should set the tooltip here, but the current api
-       * doesn't allow it before the item is added to a toolbar. 
-       */
+
+      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (action),
+                                            "tooltip");
+      gtk_action_sync_tooltip (action, pspec, proxy);
+
       g_signal_connect_object (action, "notify::visible-horizontal",
 			       G_CALLBACK (gtk_action_sync_property), 
 			       proxy, 0);
@@ -925,9 +929,7 @@ connect_proxy (GtkAction     *action,
 			"use_underline", TRUE,
 			"stock_id", action->private_data->stock_id,
 			NULL);
-	  /* FIXME: we should set the tooltip here, but the current api
-	   * doesn't allow it before the item is added to a toolbar. 
-	   */
+
 	  g_signal_connect_object (action, "notify::short-label",
 				   G_CALLBACK (gtk_action_sync_short_label),
 				   proxy, 0);      
@@ -1574,3 +1576,6 @@ gtk_action_disconnect_accelerator (GtkAction *action)
     gtk_accel_group_disconnect (action->private_data->accel_group,
 				action->private_data->accel_closure);
 }
+
+#define __GTK_ACTION_C__
+#include "gtkaliasdef.c"
