@@ -27,6 +27,7 @@
 #include <config.h>
 #include "gtkbox.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   PROP_0,
@@ -711,3 +712,6 @@ gtk_box_forall (GtkContainer *container,
 	(* callback) (child->widget, callback_data);
     }
 }
+
+#define __GTK_BOX_C__
+#include "gtkaliasdef.c"
