@@ -44,6 +44,7 @@
 #include "gtksignal.h"
 #include "gtkmarshalers.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 /* Size of checks and gray levels for alpha compositing checkerboard */
 #define CHECK_SIZE  4
@@ -134,7 +135,7 @@ static void gtk_color_button_drag_data_received (GtkWidget        *widget,
 static gpointer parent_class = NULL;
 static guint color_button_signals[LAST_SIGNAL] = { 0 };
 
-static GtkTargetEntry drop_types[] = { { "application/x-color", 0, 0 } };
+static const GtkTargetEntry drop_types[] = { { "application/x-color", 0, 0 } };
 
 GType
 gtk_color_button_get_type (void)
@@ -612,7 +613,7 @@ gtk_color_button_finalize (GObject *object)
   GtkColorButton *color_button = GTK_COLOR_BUTTON (object);
 
   if (color_button->priv->gc != NULL)
-    g_object_unref (G_OBJECT (color_button->priv->gc));
+    g_object_unref (color_button->priv->gc);
   color_button->priv->gc = NULL;
 
   if (color_button->priv->cs_dialog != NULL)
@@ -882,7 +883,7 @@ gtk_color_button_set_use_alpha (GtkColorButton *color_button,
       render (color_button);
       gtk_widget_queue_draw (color_button->priv->drawing_area);
 
-      g_object_notify (G_OBJECT (color_button), "use_alpha");
+      g_object_notify (G_OBJECT (color_button), "use-alpha");
     }
 }
 
@@ -1008,3 +1009,6 @@ gtk_color_button_get_property (GObject    *object,
       break;
     }
 }
+
+#define __GTK_COLOR_BUTTON_C__
+#include "gtkaliasdef.c"
