@@ -23,7 +23,6 @@
 
 #include <config.h>
 
-#include "gtkalias.h"
 #include "gtkexpander.h"
 
 #include "gtklabel.h"
@@ -33,6 +32,7 @@
 #include "gtkintl.h"
 #include "gtkprivate.h"
 #include <gdk/gdkkeysyms.h>
+#include "gtkalias.h"
 
 #define GTK_EXPANDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_EXPANDER, GtkExpanderPrivate))
 
@@ -1492,7 +1492,7 @@ gtk_expander_set_use_underline (GtkExpander *expander,
       if (priv->label_widget && GTK_IS_LABEL (priv->label_widget))
 	gtk_label_set_use_underline (GTK_LABEL (priv->label_widget), use_underline);
 
-      g_object_notify (G_OBJECT (expander), "use_underline");
+      g_object_notify (G_OBJECT (expander), "use-underline");
     }
 }
 
@@ -1546,7 +1546,7 @@ gtk_expander_set_use_markup (GtkExpander *expander,
       if (priv->label_widget && GTK_IS_LABEL (priv->label_widget))
 	gtk_label_set_use_markup (GTK_LABEL (priv->label_widget), use_markup);
 
-      g_object_notify (G_OBJECT (expander), "use_markup");
+      g_object_notify (G_OBJECT (expander), "use-markup");
     }
 }
 
@@ -1617,7 +1617,7 @@ gtk_expander_set_label_widget (GtkExpander *expander,
     gtk_widget_queue_resize (GTK_WIDGET (expander));
 
   g_object_freeze_notify (G_OBJECT (expander));
-  g_object_notify (G_OBJECT (expander), "label_widget");
+  g_object_notify (G_OBJECT (expander), "label-widget");
   g_object_notify (G_OBJECT (expander), "label");
   g_object_thaw_notify (G_OBJECT (expander));
 }
@@ -1640,3 +1640,6 @@ gtk_expander_get_label_widget (GtkExpander *expander)
 
   return expander->priv->label_widget;
 }
+
+#define __GTK_EXPANDER_C__
+#include "gtkaliasdef.c"
