@@ -83,7 +83,6 @@ struct _GtkIconThemePrivate
    */
   GList *themes;
   GHashTable *unthemed_icons;
-  GList *unthemed_icons_caches;
   
   /* Note: The keys of this hashtable are owned by the
    * themedir and unthemed hashtables.
@@ -97,6 +96,8 @@ struct _GtkIconThemePrivate
   /* time when we last stat:ed for theme changes */
   long last_stat_time;
   GList *dir_mtimes;
+
+  gulong reset_styles_idle;
 };
 
 struct _GtkIconInfo
@@ -581,7 +582,6 @@ gtk_icon_theme_init (GtkIconTheme *icon_theme)
   priv->themes_valid = FALSE;
   priv->themes = NULL;
   priv->unthemed_icons = NULL;
-  priv->unthemed_icons_caches = NULL;
   
   priv->pixbuf_supports_svg = pixbuf_supports_svg ();
 }
@@ -597,6 +597,30 @@ free_dir_mtime (IconThemeDirMtime *dir_mtime)
 
 }
 
+static gboolean
+reset_styles_idle (gpointer user_data)
+{
+  GtkIconTheme *icon_theme;
+  GtkIconThemePrivate *priv;
+
+  GDK_THREADS_ENTER ();
+
+  icon_theme = GTK_ICON_THEME (user_data);
+  priv = icon_theme->priv;
+
+  if (priv->screen && priv->is_screen_singleton)
+    {
+      GtkSettings *settings = gtk_settings_get_for_screen (priv->screen);
+      gtk_rc_reset_styles (settings);
+    }
+
+  priv->reset_styles_idle = 0;
+
+  GDK_THREADS_LEAVE ();
+
+  return FALSE;
+}
+
 static void
 do_theme_change (GtkIconTheme *icon_theme)
 {
@@ -607,21 +631,9 @@ do_theme_change (GtkIconTheme *icon_theme)
   blow_themes (icon_theme);
   g_signal_emit (icon_theme, signal_changed, 0);
   
-  if (priv->screen && priv->is_screen_singleton)
-    {
-      GtkSettings *settings = gtk_settings_get_for_screen (priv->screen);
-      gtk_rc_reset_styles (settings);
-    }
-}
-
-static void 
-free_cache (gpointer data, 
-	    gpointer user_data)
-{
-  GtkIconCache *cache = (GtkIconCache *)data;
-
-  if (cache)
-    _gtk_icon_cache_unref (cache);
+  if (!priv->reset_styles_idle)
+    priv->reset_styles_idle = 
+      g_idle_add (reset_styles_idle, icon_theme);
 }
 
 static void
@@ -637,13 +649,9 @@ blow_themes (GtkIconTheme *icon_theme)
       g_list_foreach (priv->dir_mtimes, (GFunc)free_dir_mtime, NULL);
       g_list_free (priv->dir_mtimes);
       g_hash_table_destroy (priv->unthemed_icons);
-      if (priv->unthemed_icons_caches)
-	g_list_foreach (priv->unthemed_icons_caches, free_cache, NULL);
-      g_list_free (priv->unthemed_icons_caches);
     }
   priv->themes = NULL;
   priv->unthemed_icons = NULL;
-  priv->unthemed_icons_caches = NULL;
   priv->dir_mtimes = NULL;
   priv->all_icons = NULL;
   priv->themes_valid = FALSE;
@@ -659,6 +667,12 @@ gtk_icon_theme_finalize (GObject *object)
   icon_theme = GTK_ICON_THEME (object);
   priv = icon_theme->priv;
 
+  if (priv->reset_styles_idle)
+    {
+      g_source_remove (priv->reset_styles_idle);
+      priv->reset_styles_idle = 0;
+    }
+
   unset_screen (icon_theme);
 
   g_free (priv->current_theme);
@@ -1010,6 +1024,8 @@ load_themes (GtkIconTheme *icon_theme)
   UnthemedIcon *unthemed_icon;
   IconSuffix old_suffix, new_suffix;
   GTimeVal tv;
+  IconThemeDirMtime *dir_mtime;
+  struct stat stat_buf;
   
   priv = icon_theme->priv;
 
@@ -1027,18 +1043,21 @@ load_themes (GtkIconTheme *icon_theme)
 
   for (base = 0; base < icon_theme->priv->search_path_len; base++)
     {
-      GtkIconCache *cache;
       dir = icon_theme->priv->search_path[base];
 
-      cache = _gtk_icon_cache_new_for_path (dir);
+      dir_mtime = g_new (IconThemeDirMtime, 1);
+      dir_mtime->cache = _gtk_icon_cache_new_for_path (dir);
+      dir_mtime->dir = g_strdup (dir);
+      if (g_stat (dir, &stat_buf) == 0 && S_ISDIR (stat_buf.st_mode))
+	dir_mtime->mtime = stat_buf.st_mtime;
+      else
+	dir_mtime->mtime = 0;
 
-      if (cache != NULL)
-	{
-	  priv->unthemed_icons_caches = g_list_prepend (priv->unthemed_icons_caches, cache);
+      priv->dir_mtimes = g_list_append (priv->dir_mtimes, dir_mtime);
+
+      if (dir_mtime->cache != NULL)
+	continue;
 
-	  continue;
-	}
-	   
       gdir = g_dir_open (dir, 0, NULL);
 
       if (gdir == NULL)
@@ -1366,14 +1385,6 @@ gtk_icon_theme_has_icon (GtkIconTheme *icon_theme,
 	return TRUE;
     }
 
-  for (l = priv->unthemed_icons_caches; l; l = l->next)
-    {
-      GtkIconCache *cache = (GtkIconCache *)l->data;
-
-      if (_gtk_icon_cache_has_icon (cache, icon_name))
-	return TRUE;
-    }
-
   if (g_hash_table_lookup_extended (priv->all_icons,
 				    icon_name, NULL, NULL))
     return TRUE;
