@@ -28,6 +28,7 @@
 #include <gdk/gdk.h>
 #include "gtkinvisible.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   PROP_0,
@@ -347,3 +348,5 @@ gtk_invisible_constructor (GType                  type,
   return object;
 }
 
+#define __GTK_INVISIBLE_C__
+#include "gtkaliasdef.c"
