@@ -20,7 +20,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkmenutoolbutton.h"
 #include "gtkintl.h"
 #include "gtktogglebutton.h"
@@ -29,6 +28,7 @@
 #include "gtkvbox.h"
 #include "gtkmenu.h"
 #include "gtkmain.h"
+#include "gtkalias.h"
 
 
 #define GTK_MENU_TOOL_BUTTON_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GTK_TYPE_MENU_TOOL_BUTTON, GtkMenuToolButtonPrivate))
@@ -161,6 +161,9 @@ gtk_menu_tool_button_construct_contents (GtkMenuToolButton *button)
   gtk_container_add (GTK_CONTAINER (button), priv->box);
   gtk_widget_show_all (priv->box);
 
+  gtk_button_set_relief (GTK_BUTTON (priv->arrow_button),
+			 gtk_tool_item_get_relief_style (GTK_TOOL_ITEM (button)));
+  
   gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
@@ -409,7 +412,6 @@ gtk_menu_tool_button_init (GtkMenuToolButton *button)
 
   arrow_button = gtk_toggle_button_new ();
   arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
-  gtk_button_set_relief (GTK_BUTTON (arrow_button), GTK_RELIEF_NONE);
   gtk_container_add (GTK_CONTAINER (arrow_button), arrow);
   gtk_box_pack_end (GTK_BOX (box), arrow_button,
                     FALSE, FALSE, 0);
@@ -586,7 +588,7 @@ gtk_menu_tool_button_get_menu (GtkMenuToolButton *button)
 }
 
 /**
- * gtk_menu_tool_set_arrow_tooltip:
+ * gtk_menu_tool_button_set_arrow_tooltip:
  * @button: a #GtkMenuToolButton
  * @tooltips: the #GtkTooltips object to be used
  * @tip_text: text to be used as tooltip text for tool_item
@@ -609,3 +611,5 @@ gtk_menu_tool_button_set_arrow_tooltip (GtkMenuToolButton *button,
   gtk_tooltips_set_tip (tooltips, button->priv->arrow_button, tip_text, tip_private);
 }
 
+#define __GTK_MENU_TOOL_BUTTON_C__
+#include "gtkaliasdef.c"
