@@ -25,9 +25,9 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkruler.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   PROP_0,
@@ -292,7 +292,7 @@ gtk_ruler_set_range (GtkRuler *ruler,
   if (ruler->max_size != max_size)
     {
       ruler->max_size = max_size;
-      g_object_notify (G_OBJECT (ruler), "max_size");
+      g_object_notify (G_OBJECT (ruler), "max-size");
     }
   g_object_thaw_notify (G_OBJECT (ruler));
 
@@ -477,3 +477,6 @@ gtk_ruler_make_pixmap (GtkRuler *ruler)
       gdk_gc_set_exposures (ruler->non_gr_exp_gc, FALSE);
     }
 }
+
+#define __GTK_RULER_C__
+#include "gtkaliasdef.c"
