@@ -181,7 +181,7 @@ gtk_scale_class_init (GtkScaleClass *class)
                                    g_param_spec_boolean ("draw_value",
 							 P_("Draw Value"),
 							 P_("Whether the current value is displayed as a string next to the slider"),
-							 FALSE,
+							 TRUE,
 							 G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
@@ -190,7 +190,7 @@ gtk_scale_class_init (GtkScaleClass *class)
 						      P_("Value Position"),
 						      P_("The position in which the current value is displayed"),
 						      GTK_TYPE_POSITION_TYPE,
-						      GTK_POS_LEFT,
+						      GTK_POS_TOP,
 						      G_PARAM_READWRITE));
 
   gtk_widget_class_install_style_property (widget_class,
@@ -265,10 +265,10 @@ gtk_scale_class_init (GtkScaleClass *class)
   add_slider_binding (binding_set, GDK_KP_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_DOWN);
    
-  add_slider_binding (binding_set, GDK_Page_Up, 0,
+  add_slider_binding (binding_set, GDK_Page_Up, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_LEFT);
 
-  add_slider_binding (binding_set, GDK_KP_Page_Up, 0,
+  add_slider_binding (binding_set, GDK_KP_Page_Up, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_LEFT);  
 
   add_slider_binding (binding_set, GDK_Page_Up, 0,
@@ -277,10 +277,10 @@ gtk_scale_class_init (GtkScaleClass *class)
   add_slider_binding (binding_set, GDK_KP_Page_Up, 0,
                       GTK_SCROLL_PAGE_UP);
   
-  add_slider_binding (binding_set, GDK_Page_Down, 0,
+  add_slider_binding (binding_set, GDK_Page_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_RIGHT);
 
-  add_slider_binding (binding_set, GDK_KP_Page_Down, 0,
+  add_slider_binding (binding_set, GDK_KP_Page_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_RIGHT);
 
   add_slider_binding (binding_set, GDK_Page_Down, 0,
