@@ -21,7 +21,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtktoolbutton.h"
 #include "gtkbutton.h"
 #include "gtkhbox.h"
@@ -34,6 +33,7 @@
 #include "gtkintl.h"
 #include "gtktoolbar.h"
 #include "gtkiconfactory.h"
+#include "gtkalias.h"
 
 #include <string.h>
 
@@ -262,14 +262,6 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
   GtkIconSize icon_size;
   GtkWidget *box = NULL;
 
-  if (gtk_tool_item_get_proxy_menu_item (tool_item, MENU_ID))
-    {
-      /* Remove item, so it will be recreated on the next
-       * create_proxy_menu_item()
-       */
-      gtk_tool_item_set_proxy_menu_item (tool_item, MENU_ID, NULL);
-    }
-  
   if (button->priv->icon_widget && button->priv->icon_widget->parent)
     {
       gtk_container_remove (GTK_CONTAINER (button->priv->icon_widget->parent),
@@ -304,12 +296,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
     {
       need_label = TRUE;
     }
-  
-  if (style != GTK_TOOLBAR_ICONS &&
-      ((style != GTK_TOOLBAR_BOTH_HORIZ ||
-	gtk_tool_item_get_is_important (GTK_TOOL_ITEM (button)))))
-    need_label = TRUE;
-  
+
   if (need_label)
     {
       if (button->priv->label_widget)
@@ -360,7 +347,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	  
 	  if (GTK_IS_IMAGE (icon))
 	    {
-	      g_object_set (G_OBJECT (button->priv->icon_widget),
+	      g_object_set (button->priv->icon_widget,
 			    "icon-size", icon_size,
 			    NULL);
 	    }
@@ -407,6 +394,8 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
   gtk_button_set_relief (GTK_BUTTON (button->priv->button),
 			 gtk_tool_item_get_relief_style (GTK_TOOL_ITEM (button)));
 
+  gtk_tool_item_rebuild_menu (tool_item);
+  
   gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
@@ -493,10 +482,10 @@ gtk_tool_button_finalize (GObject *object)
     g_free (button->priv->label_text);
 
   if (button->priv->label_widget)
-    g_object_unref (G_OBJECT (button->priv->label_widget));
+    g_object_unref (button->priv->label_widget);
 
   if (button->priv->icon_widget)
-    g_object_unref (G_OBJECT (button->priv->icon_widget));
+    g_object_unref (button->priv->icon_widget);
   
   parent_class->finalize (object);
 }
@@ -750,7 +739,7 @@ gtk_tool_button_set_use_underline (GtkToolButton *button,
 
       gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
 
-      g_object_notify (G_OBJECT (button), "use_underline");
+      g_object_notify (G_OBJECT (button), "use-underline");
     }
 }
 
@@ -798,7 +787,7 @@ gtk_tool_button_set_stock_id (GtkToolButton *button,
   button->priv->stock_id = g_strdup (stock_id);
   gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
   
-  g_object_notify (G_OBJECT (button), "stock_id");
+  g_object_notify (G_OBJECT (button), "stock-id");
 
   g_free (old_stock_id);
 }
@@ -850,7 +839,7 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
 				    button->priv->icon_widget);
 	    }
 
-	  g_object_unref (G_OBJECT (button->priv->icon_widget));
+	  g_object_unref (button->priv->icon_widget);
 	}
       
       if (icon_widget)
@@ -863,7 +852,7 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
 
       gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
       
-      g_object_notify (G_OBJECT (button), "icon_widget");
+      g_object_notify (G_OBJECT (button), "icon-widget");
     }
 }
 
@@ -910,7 +899,7 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
 
       gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
       
-      g_object_notify (G_OBJECT (button), "label_widget");
+      g_object_notify (G_OBJECT (button), "label-widget");
     }
 }
 
@@ -960,3 +949,5 @@ _gtk_tool_button_get_button (GtkToolButton *button)
   return button->priv->button;
 }
 
+#define __GTK_TOOL_BUTTON_C__
+#include "gtkaliasdef.c"
