@@ -21,13 +21,13 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtktoolitem.h"
 #include "gtkmarshalers.h"
 #include "gtktoolbar.h"
 #include "gtkseparatormenuitem.h"
 #include "gtkintl.h"
 #include "gtkmain.h"
+#include "gtkalias.h"
 
 #include <string.h>
 
@@ -787,7 +787,7 @@ gtk_tool_item_set_is_important (GtkToolItem *tool_item, gboolean is_important)
 
       gtk_widget_queue_resize (GTK_WIDGET (tool_item));
 
-      g_object_notify (G_OBJECT (tool_item), "is_important");
+      g_object_notify (G_OBJECT (tool_item), "is-important");
     }
 }
 
@@ -914,7 +914,7 @@ gtk_tool_item_set_visible_horizontal (GtkToolItem *toolitem,
     {
       toolitem->priv->visible_horizontal = visible_horizontal;
 
-      g_object_notify (G_OBJECT (toolitem), "visible_horizontal");
+      g_object_notify (G_OBJECT (toolitem), "visible-horizontal");
 
       gtk_widget_queue_resize (GTK_WIDGET (toolitem));
     }
@@ -965,7 +965,7 @@ gtk_tool_item_set_visible_vertical (GtkToolItem *toolitem,
     {
       toolitem->priv->visible_vertical = visible_vertical;
 
-      g_object_notify (G_OBJECT (toolitem), "visible_vertical");
+      g_object_notify (G_OBJECT (toolitem), "visible-vertical");
 
       gtk_widget_queue_resize (GTK_WIDGET (toolitem));
     }
@@ -1104,7 +1104,7 @@ gtk_tool_item_set_proxy_menu_item (GtkToolItem *tool_item,
   if (tool_item->priv->menu_item != menu_item)
     {
       if (tool_item->priv->menu_item)
-	g_object_unref (G_OBJECT (tool_item->priv->menu_item));
+	g_object_unref (tool_item->priv->menu_item);
       
       if (menu_item)
 	{
@@ -1139,3 +1139,5 @@ _gtk_tool_item_toolbar_reconfigured (GtkToolItem *tool_item)
   gtk_widget_queue_resize (GTK_WIDGET (tool_item));
 }
 
+#define __GTK_TOOL_ITEM_C__
+#include "gtkaliasdef.c"
