@@ -123,7 +123,6 @@ GtkTreePath *gtk_tree_path_new_from_string  (const gchar       *path);
 GtkTreePath *gtk_tree_path_new_from_indices (gint               first_index,
 					     ...);
 gchar       *gtk_tree_path_to_string        (GtkTreePath       *path);
-#define gtk_tree_path_new_root() gtk_tree_path_new_first()
 GtkTreePath *gtk_tree_path_new_first        (void);
 void         gtk_tree_path_append_index     (GtkTreePath       *path,
 					     gint               index_);
@@ -133,7 +132,7 @@ gint         gtk_tree_path_get_depth        (GtkTreePath       *path);
 gint        *gtk_tree_path_get_indices      (GtkTreePath       *path);
 void         gtk_tree_path_free             (GtkTreePath       *path);
 GtkTreePath *gtk_tree_path_copy             (const GtkTreePath *path);
-GType        gtk_tree_path_get_type         (void);
+GType        gtk_tree_path_get_type         (void) G_GNUC_CONST;
 gint         gtk_tree_path_compare          (const GtkTreePath *a,
 					     const GtkTreePath *b);
 void         gtk_tree_path_next             (GtkTreePath       *path);
@@ -146,12 +145,16 @@ gboolean     gtk_tree_path_is_ancestor      (GtkTreePath       *path,
 gboolean     gtk_tree_path_is_descendant    (GtkTreePath       *path,
                                              GtkTreePath       *ancestor);
 
+#ifndef GTK_DISABLE_DEPRECATED
+#define gtk_tree_path_new_root() gtk_tree_path_new_first()
+#endif /* !GTK_DISABLE_DEPRECATED */
+
 /* Row reference (an object that tracks model changes so it refers to the same
  * row always; a path refers to a position, not a fixed row).  You almost always
  * want to call gtk_tree_row_reference_new.
  */
 
-GType                gtk_tree_row_reference_get_type (void);
+GType                gtk_tree_row_reference_get_type (void) G_GNUC_CONST;
 GtkTreeRowReference *gtk_tree_row_reference_new       (GtkTreeModel        *model,
 						       GtkTreePath         *path);
 GtkTreeRowReference *gtk_tree_row_reference_new_proxy (GObject             *proxy,
@@ -175,7 +178,7 @@ void                 gtk_tree_row_reference_reordered (GObject     *proxy,
 /* GtkTreeIter operations */
 GtkTreeIter *     gtk_tree_iter_copy             (GtkTreeIter  *iter);
 void              gtk_tree_iter_free             (GtkTreeIter  *iter);
-GType             gtk_tree_iter_get_type         (void);
+GType             gtk_tree_iter_get_type         (void) G_GNUC_CONST;
 
 GType             gtk_tree_model_get_type        (void) G_GNUC_CONST;
 GtkTreeModelFlags gtk_tree_model_get_flags       (GtkTreeModel *tree_model);
@@ -193,7 +196,6 @@ gboolean          gtk_tree_model_get_iter_from_string (GtkTreeModel *tree_model,
 						       const gchar  *path_string);
 gchar *           gtk_tree_model_get_string_from_iter (GtkTreeModel *tree_model,
                                                        GtkTreeIter  *iter);
-#define gtk_tree_model_get_iter_root(tree_model, iter) gtk_tree_model_get_iter_first(tree_model, iter)
 gboolean          gtk_tree_model_get_iter_first  (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter);
 GtkTreePath *     gtk_tree_model_get_path        (GtkTreeModel *tree_model,
@@ -235,6 +237,10 @@ void              gtk_tree_model_foreach         (GtkTreeModel            *model
 						  gpointer                 user_data);
 
 
+#ifndef GTK_DISABLE_DEPRECATED
+#define gtk_tree_model_get_iter_root(tree_model, iter) gtk_tree_model_get_iter_first(tree_model, iter)
+#endif /* !GTK_DISABLE_DEPRECATED */
+
 /* Signals */
 void gtk_tree_model_row_changed           (GtkTreeModel *tree_model,
 					   GtkTreePath  *path,
