@@ -24,7 +24,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtkvpaned.h"
+#include "gtkalias.h"
 
 static void     gtk_vpaned_class_init     (GtkVPanedClass *klass);
 static void     gtk_vpaned_init           (GtkVPaned      *vpaned);
@@ -35,26 +37,28 @@ static void     gtk_vpaned_size_allocate  (GtkWidget      *widget,
 
 static gpointer parent_class;
 
-GtkType
+GType
 gtk_vpaned_get_type (void)
 {
-  static GtkType vpaned_type = 0;
+  static GType vpaned_type = 0;
 
   if (!vpaned_type)
     {
-      static const GtkTypeInfo vpaned_info =
+      static const GTypeInfo vpaned_info =
       {
-	"GtkVPaned",
-	sizeof (GtkVPaned),
 	sizeof (GtkVPanedClass),
-	(GtkClassInitFunc) gtk_vpaned_class_init,
-	(GtkObjectInitFunc) gtk_vpaned_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_vpaned_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkVPaned),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_vpaned_init,
       };
 
-      vpaned_type = gtk_type_unique (GTK_TYPE_PANED, &vpaned_info);
+      vpaned_type = g_type_register_static (GTK_TYPE_PANED, "GtkVPaned",
+					    &vpaned_info, 0);
     }
 
   return vpaned_type;
@@ -65,7 +69,7 @@ gtk_vpaned_class_init (GtkVPanedClass *class)
 {
   GtkWidgetClass *widget_class;
 
-  parent_class = gtk_type_class (GTK_TYPE_PANED);
+  parent_class = g_type_class_peek_parent (class);
   
   widget_class = (GtkWidgetClass *) class;
 
@@ -91,7 +95,7 @@ gtk_vpaned_new (void)
 {
   GtkVPaned *vpaned;
 
-  vpaned = gtk_type_new (GTK_TYPE_VPANED);
+  vpaned = g_object_new (GTK_TYPE_VPANED, NULL);
 
   return GTK_WIDGET (vpaned);
 }
@@ -182,11 +186,11 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
 	}
 
       child1_allocation.width = child2_allocation.width = MAX (1, (gint) allocation->width - border_width * 2);
-      child1_allocation.height = paned->child1_size;
+      child1_allocation.height = MAX (1, paned->child1_size);
       child1_allocation.x = child2_allocation.x = widget->allocation.x + border_width;
       child1_allocation.y = widget->allocation.y + border_width;
       
-      child2_allocation.y = child1_allocation.y + child1_allocation.height + paned->handle_pos.height;
+      child2_allocation.y = child1_allocation.y + paned->child1_size + paned->handle_pos.height;
       child2_allocation.height = MAX (1, widget->allocation.y + widget->allocation.height - child2_allocation.y - border_width);
       
       /* Now allocate the childen, making sure, when resizing not to
@@ -210,7 +214,12 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
 
       if (GTK_WIDGET_REALIZED (widget))      
 	gdk_window_hide (paned->handle);
-	  
+
+      if (paned->child1)
+	gtk_widget_set_child_visible (paned->child1, TRUE);
+      if (paned->child2)
+	gtk_widget_set_child_visible (paned->child2, TRUE);
+
       child_allocation.x = widget->allocation.x + border_width;
       child_allocation.y = widget->allocation.y + border_width;
       child_allocation.width = MAX (1, allocation->width - 2 * border_width);
@@ -222,3 +231,6 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
 	gtk_widget_size_allocate (paned->child2, &child_allocation);
     }
 }
+
+#define __GTK_VPANED_C__
+#include "gtkaliasdef.c"
