@@ -24,9 +24,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <math.h>
 #include <string.h>
 #include "gtkvruler.h"
+#include "gtkalias.h"
 
 #include <glib/gprintf.h>
 
@@ -327,3 +329,6 @@ gtk_vruler_draw_pos (GtkRuler *ruler)
 	}
     }
 }
+
+#define __GTK_VRULER_C__
+#include "gtkaliasdef.c"
